# coding:utf-8

from grp import getgrgid
import os
from pwd import getpwuid
from tempfile import TemporaryDirectory
import unittest

from xkits import safile
from xkits import stfile


class test_stfile(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.tdir = TemporaryDirectory()
        cls.path = os.path.join(cls.tdir.name, "unittest")
        with open(cls.path, "w") as whdl:
            whdl.write("unittest")
        cls.file = stfile(cls.path)
        cls.username = getpwuid(os.getuid()).pw_name
        cls.groupname = getgrgid(os.getgid()).gr_name

    @classmethod
    def tearDownClass(cls):
        cls.tdir.cleanup()

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_check_file(self):
        self.assertEqual(self.file.path, self.path)
        self.assertIsInstance(self.file.stat, os.stat_result)
        self.file.username = self.username
        self.file.groupname = self.groupname
        self.assertEqual(self.file.uid, os.getuid())
        self.assertEqual(self.file.gid, os.getgid())
        self.assertEqual(self.file.username, self.username)
        self.assertEqual(self.file.groupname, self.groupname)
        self.file.chmod("777")
        self.assertEqual(self.file.mode, "100777")
        self.assertEqual(self.file.human_file_type, "-")
        self.assertEqual(self.file.human_mode, "-rwxrwxrwx")
        self.assertEqual(self.file.human_owner_permissions, "rwx")
        self.assertEqual(self.file.human_group_permissions, "rwx")
        self.assertEqual(self.file.human_other_permissions, "rwx")
        self.file.chown(str(self.file.uid))
        self.file.chown(self.username)
        self.file.chown(self.file.uid)
        self.file.chgrp(str(self.file.gid))
        self.file.chgrp(self.groupname)
        self.file.chgrp(self.file.gid)


class test_safile(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.text = "ZbbwpP4%oSwYxP=t+LkyXXzqL9fE8!"

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_backup_and_restore(self):
        with TemporaryDirectory() as thdl:
            path = os.path.join(thdl, "test")
            self.assertTrue(safile.create_backup(path))
            with open(path, "w") as whdl:
                whdl.write(self.text)
            self.assertTrue(safile.create_backup(path, copy=True))
            self.assertTrue(safile.create_backup(path))
            with open(path, "w") as whdl:
                whdl.write("unittest")
            self.assertTrue(safile.restore(path))
            with open(path, "r") as rhdl:
                self.assertEqual(rhdl.read(), self.text)
            self.assertTrue(safile.create_backup(path, copy=False))
            self.assertTrue(safile.delete_backup(path))


if __name__ == "__main__":
    unittest.main()



# coding:utf-8

from argparse import Namespace
import sys
import unittest

import mock

from xkits import argp


class test_argp(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        self.argp = argp("xkits-test")

    def tearDown(self):
        pass

    @mock.patch.object(sys, "exit")
    def test_help_optional_h(self, mock_exit: mock.Mock):
        mock_exit.side_effect = [Exception("xkits-test")]
        self.assertRaises(Exception, self.argp.parse_args, "-h".split())
        mock_exit.assert_called_once_with(0)

    @mock.patch.object(sys, "exit")
    def test_help_optional_help(self, mock_exit: mock.Mock):
        mock_exit.side_effect = [Exception("xkits-test")]
        self.assertRaises(Exception, self.argp.parse_args, "--help".split())
        mock_exit.assert_called_once_with(0)

    def test_argument_group(self):
        group1 = self.argp.argument_group("test")
        group2 = self.argp.argument_group("test")
        self.assertEqual(group1, group2)

    def test_filter_optional_name(self):
        self.argp.add_opt_on("--debug")
        ret = self.argp.filter_optional_name("-d", "--debug")
        self.assertSequenceEqual(ret, ["-d"])

    def test_add_pos(self):
        self.argp.add_pos("path", type=str, nargs="+")
        args = self.argp.parse_args("test.py 123.txt".split())
        self.assertIsInstance(args, Namespace)
        self.assertIsInstance(args.path, list)
        self.assertEqual(args.path, ["test.py", "123.txt"])

    def test_add_opt(self):
        self.argp.add_opt("-v", "--value", type=int, nargs=1)
        args = self.argp.parse_args("--value 100".split())
        self.assertIsInstance(args, Namespace)
        self.assertIsInstance(args.value, int)
        self.assertEqual(args.value, 100)

    def test_add_opt_on_close(self):
        self.argp.add_opt_on("--dry-run")
        args = self.argp.parse_args("".split())
        self.assertIsInstance(args, Namespace)
        self.assertIsInstance(args.dry_run, bool)
        self.assertEqual(args.dry_run, False)

    def test_add_opt_on_open(self):
        self.argp.add_opt_on("--dry-run")
        args = self.argp.parse_args("--dry-run".split())
        self.assertIsInstance(args, Namespace)
        self.assertIsInstance(args.dry_run, bool)
        self.assertEqual(args.dry_run, True)

    def test_add_opt_off_close(self):
        self.argp.add_opt_off("--no-check")
        args = self.argp.parse_args("".split())
        self.assertIsInstance(args, Namespace)
        self.assertIsInstance(args.no_check, bool)
        self.assertEqual(args.no_check, True)

    def test_add_opt_off_open(self):
        self.argp.add_opt_off("--no-check")
        args = self.argp.parse_args("--no-check".split())
        self.assertIsInstance(args, Namespace)
        self.assertIsInstance(args.no_check, bool)
        self.assertEqual(args.no_check, False)

    def test_add_subparsers(self):
        _sub = self.argp.add_subparsers(dest="sub")
        _arg = _sub.add_parser("list")
        _arg.add_opt_on("-d", "--debug")
        args = self.argp.parse_args("list --debug".split())
        self.assertIsInstance(args, Namespace)
        self.assertIsInstance(args.sub, str)
        self.assertEqual(args.sub, "list")
        self.assertIsInstance(args.debug, bool)
        self.assertEqual(args.debug, True)


if __name__ == "__main__":
    unittest.main()



# coding:utf-8

from time import sleep
import unittest

from xkits import hourglass


class test_execute(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        pass

    def tearDown(self):
        pass

    @hourglass(0.5)
    def fake_hourglass(self, value: float = 1.0):
        return sleep(value)

    def test_hourglass(self):
        self.assertRaises(TimeoutError, self.fake_hourglass)
        self.assertIsNone(self.fake_hourglass(0.1))


if __name__ == "__main__":
    unittest.main()



# coding:utf-8

import unittest

import mock
from requests import Session

from xkits import Page
from xkits import ProxyProtocol
from xkits import ProxySession
from xkits import Site
from xkits import sitepage


class test_proxy_session(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.host: str = "127.0.0.1"
        cls.port: int = 12345

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_invalid_proxy(self):
        self.assertRaises(ValueError, ProxySession,
                          ProxyProtocol.HTTP.value,
                          self.host, self.port)

    def test_http_proxy(self):
        proxies = {
            "http": f"{ProxyProtocol.HTTP.value}://{self.host}:{self.port}",
            "https": f"{ProxyProtocol.HTTP.value}://{self.host}:{self.port}"
        }
        proxy_session = ProxySession.http_proxy(self.host, self.port)
        self.assertEqual(proxy_session.proxies, proxies)
        self.assertIsInstance(proxy_session, Session)

    def test_https_proxy(self):
        proxies = {
            "http": f"{ProxyProtocol.HTTPS.value}://{self.host}:{self.port}",
            "https": f"{ProxyProtocol.HTTPS.value}://{self.host}:{self.port}"
        }
        proxy_session = ProxySession.https_proxy(self.host, self.port)
        self.assertEqual(proxy_session.proxies, proxies)
        self.assertIsInstance(proxy_session, Session)

    def test_socks4_proxy(self):
        proxies = {
            "http": f"{ProxyProtocol.SOCKS4.value}://{self.host}:{self.port}",
            "https": f"{ProxyProtocol.SOCKS4.value}://{self.host}:{self.port}"
        }
        proxy_session = ProxySession.socks4_proxy(self.host, self.port)
        self.assertEqual(proxy_session.proxies, proxies)
        self.assertIsInstance(proxy_session, Session)

    def test_socks5_proxy(self):
        proxies = {
            "http": f"{ProxyProtocol.SOCKS5.value}://{self.host}:{self.port}",
            "https": f"{ProxyProtocol.SOCKS5.value}://{self.host}:{self.port}"
        }
        proxy_session = ProxySession.socks5_proxy(self.host, self.port)
        self.assertEqual(proxy_session.proxies, proxies)
        self.assertIsInstance(proxy_session, Session)


class test_site(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.base_url: str = "https://iptv-org.github.io/api"
        cls.iptv_org_api: Site = Site(cls.base_url, Session())
        cls.cate_url: str = "https://iptv-org.github.io/api/categories.json"

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_chech_str(self):
        check: str = f"website {self.iptv_org_api.base} with pages cache pool"
        self.assertEqual(str(self.iptv_org_api), check)

    def test_login(self):
        with mock.patch.object(self.iptv_org_api.session, "post") as mock_post:
            fake_response = mock.MagicMock()
            mock_post.side_effect = [fake_response]
            self.assertIs(self.iptv_org_api.login("test", {}), fake_response)

    def test_categories(self):
        page = self.iptv_org_api.page("categories.json")
        self.assertIs(self.iptv_org_api[self.cate_url], page)
        self.assertIs(self.iptv_org_api.session, page.session)
        self.assertIsInstance(page, Page)
        self.assertIsInstance(page.label, str)
        self.assertEqual(str(page), f"page object at {id(page)} url={self.cate_url}")  # noqa:E501
        with mock.patch.object(page.session, "get") as mock_get:
            fake_response = mock.MagicMock()
            fake_response.content = "unittest"
            mock_get.return_value = fake_response
            with mock.patch.object(sitepage, "open") as mock_open:
                page.save("test.json")
                mock_open.assert_called_once()
            fake_response.raise_for_status.assert_called_once_with()
            with mock.patch.object(sitepage, "BeautifulSoup") as mock_soup:
                fake_soup = mock.MagicMock()
                mock_soup.return_value = fake_soup
                self.assertIs(page.soup, fake_soup)



# coding:utf-8

from time import time
import unittest

from xkits import NamedLock
from xkits import TaskJob
from xkits import TaskPool
from xkits import ThreadPool


class test_named_lock(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.namedlock: NamedLock[str] = NamedLock()
        cls.lockname: str = "test"

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_lock(self):
        self.assertEqual(len(self.namedlock), 0)
        self.assertNotIn(self.lockname, self.namedlock)
        self.assertIsInstance(self.namedlock.lookup(self.lockname), NamedLock.LockItem)  # noqa:E501
        self.assertEqual(len(self.namedlock), 1)
        self.assertIn(self.lockname, self.namedlock)
        for lock in self.namedlock:
            with lock.lock:
                self.assertIs(self.namedlock[lock.name], lock.lock)


class test_thread_pool(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_init(self):
        pool: ThreadPool = ThreadPool(1)
        pool.cmds.stdout("unittest")
        self.assertIsInstance(pool.alive_threads, set)
        self.assertIsInstance(pool.other_threads, set)
        self.assertIsInstance(pool.other_alive_threads, set)


class test_task_pool(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_job(self):
        def handle(value: bool) -> bool:
            return value

        def result(job: TaskJob):
            return job.result

        job: TaskJob = TaskJob(1, handle, False)
        self.assertRaises(LookupError, result, job)
        self.assertLess(job.created, time())
        self.assertEqual(job.started, 0.0)
        self.assertEqual(job.stopped, 0.0)
        self.assertEqual(job.id, 1)
        self.assertTrue(job.run())
        self.assertFalse(job.result)
        self.assertLess(job.created, time())
        self.assertLess(job.started, time())
        self.assertLess(job.stopped, time())
        self.assertFalse(job.run())

    def test_task(self):
        def lock(tasker: TaskPool, index: int):
            tasker.cmds.stdout(f"{index}")
            if index % 2 == 1:
                raise Exception(f"task{index}")
        with TaskPool(8) as tasker:
            for index in range(15):
                tasker.submit(lock, tasker, index)
            tasker.barrier()
            self.assertEqual(tasker.counter, 15)
            self.assertEqual(tasker.suceess, 8)
            self.assertEqual(tasker.failure, 7)
            self.assertTrue(tasker.running)
            tasker.shutdown()
            self.assertFalse(tasker.running)
            tasker.startup()
            self.assertTrue(tasker.running)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
# coding:utf-8

import os
import shutil
import unittest

from xkits import scanner


def handler(obj: scanner.object) -> bool:
    return isinstance(obj, scanner.object)


class test_scanner(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.scanner = scanner.load(paths=[os.path.join("xkits")],
                                   exclude=[os.path.join("xkits", "test")],
                                   handler=handler)

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_load(self):
        self.assertIsInstance(scanner.load(paths=[os.path.join("scanloader")], handler=handler), scanner)  # noqa:E501

    def test_iter(self):
        for object in self.scanner:
            self.assertIsInstance(object, scanner.object)
            self.assertIsInstance(object.path, str)
            self.assertIsInstance(object.abspath, str)
            self.assertIsInstance(object.realpath, str)
            self.assertIsInstance(object.uid, int)
            self.assertIsInstance(object.gid, int)
            self.assertIsInstance(object.mode, int)
            self.assertIsInstance(object.size, int)
            self.assertIsInstance(object.ctime, float)
            self.assertIsInstance(object.atime, float)
            self.assertIsInstance(object.mtime, float)
            self.assertIsInstance(object.isdir, bool)
            self.assertIsInstance(object.isfile, bool)
            self.assertIsInstance(object.islink, bool)

            if object.isfile and not object.issym:
                self.assertIsInstance(object.md5, str)
                self.assertIsInstance(object.sha1, str)
                self.assertIsInstance(object.sha256, str)

    def test_dirs(self):
        for object in self.scanner.dirs:
            self.assertTrue(object.isdir)

    def test_files(self):
        for object in self.scanner.files:
            self.assertTrue(object.isfile)
            self.assertTrue(object.isreg)

    def test_links(self):
        for object in self.scanner.links:
            self.assertTrue(object.issym)

    def test_add_dir_object(self):
        object = scanner.object("test")
        self.scanner.add(object)
        self.assertIs(self.scanner["test"], object)

    def test_add_sym_object(self):
        path = shutil.which("python")
        if path:
            object = scanner.object(path)
            self.scanner.add(object)
            self.assertIs(self.scanner[path], object)

    def test_add_file_object(self):
        path = os.path.join("test", "requirements.txt")
        object = scanner.object(path)
        self.scanner.add(object)
        self.assertIs(self.scanner[path], object)


if __name__ == "__main__":
    unittest.main()



# coding:utf-8

import unittest

from xkits import Back
from xkits import Fore
from xkits import Style
from xkits import color
from xkits import commands


class test_safile(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_reset(self):
        self.assertEqual(str(color.reset()), Style.RESET_ALL)

    def test_add_style(self):
        text1 = color("unittest style")
        text2 = color("unittest style")
        text1.add_style(Style.BRIGHT)
        text1.add_style(Style.UNDERLINE)
        text2.style = {Style.BRIGHT, Style.UNDERLINE}
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(text1.style, text2.style)

    def test_bold_style(self):
        text1 = color("unittest bold")
        text2 = color.bold(text1)
        text1.style = {Style.BRIGHT}
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_dim_style(self):
        text1 = color("unittest dim")
        text2 = color.dim(text1)
        text1.style = {Style.DIM}
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_normal_style(self):
        text1 = color("unittest normal")
        text2 = color.normal(text1)
        text1.style = {Style.NORMAL}
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_italic_style(self):
        text1 = color("unittest italic")
        text2 = color.italic(text1)
        text1.style = {Style.ITALIC}
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_underline_style(self):
        text1 = color("unittest underline")
        text2 = color.underline(text1)
        text1.style = {Style.UNDERLINE}
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_slow_blink_style(self):
        text1 = color("unittest slow_blink")
        text2 = color.slow_blink(text1)
        text1.style = {Style.SLOWBLINK}
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_rapid_blink_style(self):
        text1 = color("unittest rapid_blink")
        text2 = color.rapid_blink(text1)
        text1.style = {Style.RAPIDBLINK}
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_invert_style(self):
        text1 = color("unittest invert")
        text2 = color.invert(text1)
        text1.style = {Style.INVERT}
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_hide_style(self):
        text1 = color("unittest hide")
        text2 = color.hide(text1)
        text1.style = {Style.HIDE}
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_strikethrough_style(self):
        text1 = color("unittest strikethrough")
        text2 = color.strikethrough(text1)
        text1.style = {Style.STRIKETHROUGH}
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_doubly_underlined_style(self):
        text1 = color("unittest doubly_underlined")
        text2 = color.doubly_underlined(text1)
        text1.style = {Style.DOUBLYUNDERLINED}
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_reveal_style(self):
        text1 = color("unittest reveal")
        text2 = color.reveal(text1)
        text1.style = {Style.REVEAL}
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_black_foreground(self):
        text1 = color("unittest black")
        text2 = color.black(text1)
        text1.foreground = Fore.BLACK
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_red_foreground(self):
        text1 = color("unittest red")
        text2 = color.red(text1)
        text1.foreground = Fore.RED
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_green_foreground(self):
        text1 = color("unittest green")
        text2 = color.green(text1)
        text1.foreground = Fore.GREEN
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_yellow_foreground(self):
        text1 = color("unittest yellow")
        text2 = color.yellow(text1)
        text1.foreground = Fore.YELLOW
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_blue_foreground(self):
        text1 = color("unittest blue")
        text2 = color.blue(text1)
        text1.foreground = Fore.BLUE
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_magenta_foreground(self):
        text1 = color("unittest magenta")
        text2 = color.magenta(text1)
        text1.foreground = Fore.MAGENTA
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_cyan_foreground(self):
        text1 = color("unittest cyan")
        text2 = color.cyan(text1)
        text1.foreground = Fore.CYAN
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_white_foreground(self):
        text1 = color("unittest white")
        text2 = color.white(text1)
        text1.foreground = Fore.WHITE
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightblack_foreground(self):
        text1 = color("unittest lightblack")
        text2 = color.lightblack(text1)
        text1.foreground = Fore.LIGHTBLACK_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightred_foreground(self):
        text1 = color("unittest lightred")
        text2 = color.lightred(text1)
        text1.foreground = Fore.LIGHTRED_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightgreen_foreground(self):
        text1 = color("unittest lightgreen")
        text2 = color.lightgreen(text1)
        text1.foreground = Fore.LIGHTGREEN_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightyellow_foreground(self):
        text1 = color("unittest lightyellow")
        text2 = color.lightyellow(text1)
        text1.foreground = Fore.LIGHTYELLOW_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightblue_foreground(self):
        text1 = color("unittest lightblue")
        text2 = color.lightblue(text1)
        text1.foreground = Fore.LIGHTBLUE_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightmagenta_foreground(self):
        text1 = color("unittest lightmagenta")
        text2 = color.lightmagenta(text1)
        text1.foreground = Fore.LIGHTMAGENTA_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightcyan_foreground(self):
        text1 = color("unittest lightcyan")
        text2 = color.lightcyan(text1)
        text1.foreground = Fore.LIGHTCYAN_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightwhite_foreground(self):
        text1 = color("unittest lightwhite")
        text2 = color.lightwhite(text1)
        text1.foreground = Fore.LIGHTWHITE_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_black_background(self):
        text1 = color("unittest black")
        text2 = color.black_back(text1)
        text1.background = Back.BLACK
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_red_background(self):
        text1 = color("unittest red")
        text2 = color.red_back(text1)
        text1.background = Back.RED
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_green_background(self):
        text1 = color("unittest green")
        text2 = color.green_back(text1)
        text1.background = Back.GREEN
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_yellow_background(self):
        text1 = color("unittest yellow")
        text2 = color.yellow_back(text1)
        text1.background = Back.YELLOW
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_blue_background(self):
        text1 = color("unittest blue")
        text2 = color.blue_back(text1)
        text1.background = Back.BLUE
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_magenta_background(self):
        text1 = color("unittest magenta")
        text2 = color.magenta_back(text1)
        text1.background = Back.MAGENTA
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_cyan_background(self):
        text1 = color("unittest cyan")
        text2 = color.cyan_back(text1)
        text1.background = Back.CYAN
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_white_background(self):
        text1 = color("unittest white")
        text2 = color.white_back(text1)
        text1.background = Back.WHITE
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightblack_background(self):
        text1 = color("unittest lightblack")
        text2 = color.lightblack_back(text1)
        text1.background = Back.LIGHTBLACK_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightred_background(self):
        text1 = color("unittest lightred")
        text2 = color.lightred_back(text1)
        text1.background = Back.LIGHTRED_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightgreen_background(self):
        text1 = color("unittest lightgreen")
        text2 = color.lightgreen_back(text1)
        text1.background = Back.LIGHTGREEN_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightyellow_background(self):
        text1 = color("unittest lightyellow")
        text2 = color.lightyellow_back(text1)
        text1.background = Back.LIGHTYELLOW_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightblue_background(self):
        text1 = color("unittest lightblue")
        text2 = color.lightblue_back(text1)
        text1.background = Back.LIGHTBLUE_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightmagenta_background(self):
        text1 = color("unittest lightmagenta")
        text2 = color.lightmagenta_back(text1)
        text1.background = Back.LIGHTMAGENTA_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightcyan_background(self):
        text1 = color("unittest lightcyan")
        text2 = color.lightcyan_back(text1)
        text1.background = Back.LIGHTCYAN_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))

    def test_lightwhite_background(self):
        text1 = color("unittest lightwhite")
        text2 = color.lightwhite_back(text1)
        text1.background = Back.LIGHTWHITE_EX
        commands().stdout(f"text1: {text1}")
        commands().stdout(f"text2: {text2}")
        self.assertEqual(str(text1), str(text2))


if __name__ == "__main__":
    unittest.main()



# coding:utf-8

from time import sleep
import unittest

from xkits import CacheAtom
from xkits import CacheData
from xkits import CacheExpired
from xkits import CacheItem
from xkits import CacheMiss
from xkits import CachePool
from xkits import NamedCache


class test_cache(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.index = "test"
        cls.value = "unit"

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_cache_atom(self):
        item = CacheAtom(self.value, 0.1)
        self.assertFalse(item.expired)
        self.assertGreater(item.down, 0)
        self.assertEqual(item.data, self.value)
        sleep(0.2)
        self.assertTrue(item.expired)
        self.assertLess(item.down, 0)
        self.assertEqual(item.data, self.value)
        item.renew(0.5)
        self.assertFalse(item.expired)
        self.assertGreater(item.down, 0)
        self.assertEqual(item.data, self.value)
        item.data = "atom"
        self.assertEqual(item.data, "atom")
        self.assertEqual(str(item), f"cache object at {id(item)}")

    def test_cache_data(self):
        def read(item: CacheData):
            return item.data
        item = CacheData(self.value, 0.1)
        sleep(0.2)
        self.assertTrue(item.expired)
        self.assertLess(item.down, 0)
        self.assertRaises(CacheExpired, read, item)
        item.data = "data"
        self.assertEqual(item.data, "data")

    def test_named_cache(self):
        item = NamedCache(self.index, self.value, 0.1)
        sleep(0.2)
        self.assertTrue(item.expired)
        self.assertLess(item.down, 0)
        self.assertEqual(item.data, self.value)
        item.data = "name"
        self.assertEqual(item.data, "name")
        self.assertEqual(str(item), f"cache object at {id(item)} name={item.name}")  # noqa:E501

    def test_cache_item(self):
        def read(item: CacheItem):
            return item.data
        item = CacheItem(self.index, self.value, 0.1)
        sleep(0.2)
        self.assertTrue(item.expired)
        self.assertLess(item.down, 0)
        self.assertRaises(CacheExpired, read, item)
        item.data = "item"
        self.assertEqual(item.data, "item")

    def test_cache_pool(self):
        def read(pool: CachePool, name: str):
            return pool[name]
        pool: CachePool[str, str] = CachePool()
        pool[self.index] = self.value
        self.assertEqual(len(pool), 1)
        self.assertEqual(pool[self.index], self.value)
        for key in pool:
            self.assertIsInstance(pool[key], str)
        del pool[self.index]
        self.assertNotIn(self.index, pool)
        self.assertRaises(CacheMiss, read, pool, self.index)
        self.assertEqual(len(pool), 0)
        self.assertEqual(str(pool), f"cache pool at {id(pool)}")

    def test_cache_pool_timeout(self):
        def read(pool: CachePool, name: str):
            return pool[name]
        pool = CachePool(0.1)
        pool[self.index] = self.value
        self.assertEqual(pool[self.index], self.value)
        sleep(0.2)
        self.assertEqual(len(pool), 1)
        self.assertRaises(CacheMiss, read, pool, self.index)
        self.assertEqual(len(pool), 0)


if __name__ == "__main__":
    unittest.main()



# coding:utf-8

from .actuator import Namespace  # noqa:F401
from .actuator import add_command  # noqa:F401
from .actuator import cmds  # noqa:F401
from .actuator import commands  # noqa:F401
from .actuator import end_command  # noqa:F401
from .actuator import pre_command  # noqa:F401
from .actuator import run_command  # noqa:F401
from .cache import CacheAtom  # noqa:F401
from .cache import CacheData  # noqa:F401
from .cache import CacheExpired  # noqa:F401
from .cache import CacheItem  # noqa:F401
from .cache import CacheLookup  # noqa:F401
from .cache import CacheMiss  # noqa:F401
from .cache import CachePool  # noqa:F401
from .cache import CacheTimeUnit  # noqa:F401
from .cache import NamedCache  # noqa:F401
from .colorful import Back  # noqa:F401
from .colorful import Fore  # noqa:F401
from .colorful import Style  # noqa:F401
from .colorful import color  # noqa:F401
from .executor import hourglass  # noqa:F401
from .parser import argp  # noqa:F401
from .safefile import safile  # noqa:F401
from .safefile import stfile  # noqa:F401
from .scanner import scanner  # noqa:F401
from .sheet import cell  # noqa:F401
from .sheet import csv  # noqa:F401
from .sheet import form  # noqa:F401
from .sheet import row  # noqa:F401
from .sheet import tabulate  # noqa:F401
from .sheet import xls_reader  # noqa:F401
from .sheet import xls_writer  # noqa:F401
from .sheet import xlsx  # noqa:F401
from .sitepage import Page  # noqa:F401
from .sitepage import PageCache  # noqa:F401
from .sitepage import ProxyProtocol  # noqa:F401
from .sitepage import ProxySession  # noqa:F401
from .sitepage import Site  # noqa:F401
from .thread import NamedLock  # noqa:F401
from .thread import TaskJob  # noqa:F401
from .thread import TaskPool  # noqa:F401
from .thread import ThreadPool  # noqa:F401
from .utils import chdir  # noqa:F401
from .utils import singleton  # noqa:F401



# coding:utf-8

import unittest

from xkits import chdir


class test_chdir(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_chdir(self):
        self.assertRaises(AssertionError, chdir().popd)
        chdir().pushd("xkits")
        chdir().pushd("unittest")
        chdir().popd()
        chdir().popd()
        self.assertRaises(AssertionError, chdir().popd)


if __name__ == "__main__":
    unittest.main()



# coding:utf-8

import os
from tempfile import TemporaryDirectory
from typing import Union
import unittest

from xkits import cell
from xkits import csv
from xkits import form
from xkits import row
from xkits import tabulate
from xkits import xls_reader
from xkits import xls_writer
from xkits import xlsx


class test_sheet(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        TemporaryDirectory()

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        self.fake_form: form[str, Union[str, int]] = form(
            "scores", ["name", "score"])
        self.fake_form.extend([["alice", 90], ["cindy", 80]])
        self.fake_form.append(["eric", 70])

    def tearDown(self):
        pass

    def test_form(self):
        row = self.fake_form.new_row(["frank", 60])
        row.extend([None, 50])
        row.append("good")
        row[4] = "bad"
        self.fake_form[2] = row
        row[0].value = "garry"
        self.assertEqual(len(row), 5)
        self.assertFalse(row[3].empty)
        self.assertEqual(str(row[1]), "60")
        self.assertEqual(row[4].value, "bad")
        self.assertEqual(self.fake_form[2], row)
        self.assertEqual(len(self.fake_form), 3)
        self.assertEqual(self.fake_form.new_map(),
                         {"name": None, "score": None})
        self.assertEqual(self.fake_form.column_no("name"), 0)
        self.assertEqual(self.fake_form.column_no("score"), 1)

    def test_form_sort(self):
        def handle(items: row[str, Union[str, int]]) -> cell[Union[str, int]]:
            return items[1]
        self.fake_form.sort(handle)

    def test_tabulate(self):
        print(tabulate(self.fake_form))

    def test_csv_header(self):
        with TemporaryDirectory() as thdl:
            path = os.path.join(thdl, "test.csv")
            csv.dump(path, self.fake_form)
            csv.load(path)

    def test_csv_no_header(self):
        self.fake_form.header = []
        with TemporaryDirectory() as thdl:
            path = os.path.join(thdl, "test.csv")
            csv.dump(path, self.fake_form)
            csv.load(path, include_header=False)

    def test_xls_header_sheet(self):
        with TemporaryDirectory() as thdl:
            path = os.path.join(thdl, "sheet", "test.xls")
            writer = xls_writer()
            writer.dump_sheet(self.fake_form)
            writer.save(path)
            reader = xls_reader(path)
            reader.load_sheet()
            self.assertEqual(reader.file, path)

    def test_xls_header_sheets(self):
        with TemporaryDirectory() as thdl:
            path = os.path.join(thdl, "sheets", "test.xls")
            writer = xls_writer()
            writer.dump_sheets([self.fake_form,
                                form("socre2", ["name", "score2"])])
            writer.save(path)
            reader = xls_reader(path)
            reader.load_sheets()
            self.assertEqual(reader.file, path)

    def test_xlsx_load_sheets(self):
        path = os.path.join("test", "example.xlsx")
        reader = xlsx(path)
        reader.load_sheets()
        self.assertEqual(reader.file, path)

    def test_xlsx_load_sheet(self):
        path = os.path.join("test", "example.xlsx")
        reader = xlsx(path)
        reader.load_sheet()


if __name__ == "__main__":
    unittest.main()



# coding:utf-8

from argparse import Namespace
from errno import ECANCELED
from errno import ENOENT
from errno import ENOTRECOVERABLE
import os
import sys
from tempfile import TemporaryDirectory
from typing import Tuple
import unittest

import mock

from xkits import add_command
from xkits import argp
from xkits import commands
from xkits import end_command
from xkits import pre_command
from xkits import run_command
from xkits.attribute import __version__


@add_command("debug", help="test logger level")
def add_cmd_debug(_arg: argp):
    commands().stdout("debug")
    _arg.add_opt_on("-d", "--debug")


@run_command(add_cmd_debug)
def run_cmd_debug(cmds: commands) -> int:
    cmds.logger.debug("debug")
    return 0


@add_command("known", help="test preparse")
def add_cmd_known(_arg: argp):
    commands().stdout("known")
    _arg.preparse_from_sys_argv()


@run_command(add_cmd_known)
def run_cmd_known(cmds: commands) -> int:
    cmds.logger.debug("known")
    return 0


@add_command("list", description="test list")
def add_cmd_list(_arg: argp):
    commands().stdout("list")


@run_command(add_cmd_list, add_cmd_debug, add_cmd_known)
def run_cmd_list(cmds: commands) -> int:
    cmds.logger.info("list")
    return 0


@add_command("incomplete", help="test incomplete")
def add_cmd_incomplete(_arg: argp):
    commands().stderr("error")


@run_command(add_cmd_incomplete)
def run_cmd_incomplete(cmds: commands) -> int:
    cmds.logger.error("incomplete")
    return -1


@add_command("keyboard", help="test KeyboardInterrupt")
def add_cmd_keyboard(_arg: argp):
    commands().stdout("keyboard")


@run_command(add_cmd_keyboard)
def run_cmd_keyboard(cmds: commands) -> int:
    cmds.logger.error("keyboard")
    raise KeyboardInterrupt


@add_command("exception", help="test BaseException")
def add_cmd_exception(_arg: argp):
    commands().stdout("exception")


@run_command(add_cmd_exception)
def run_cmd_exception(cmds: commands) -> int:
    cmds.logger.error("exception")
    raise BaseException


@add_command("prepare", help="test prepare")
def add_cmd_prepare(_arg: argp):
    commands().stdout("prepare")


@run_command(add_cmd_prepare)
def run_cmd_prepare(cmds: commands) -> int:
    return 0


@pre_command(run_cmd_prepare)
def pre_cmd_prepare(cmds: commands) -> int:
    cmds.logger.info("prepare")
    return -1


@add_command("purge", help="test purge")
def add_cmd_purge(_arg: argp):
    commands().stdout("purge")


@run_command(add_cmd_purge)
def run_cmd_purge(cmds: commands) -> int:
    return 0


@end_command(run_cmd_purge)
def end_cmd_purge(cmds: commands) -> int:
    cmds.logger.info("purge")
    return 1


@add_command("example", description="example")
def add_cmd(_arg: argp):
    commands().stdout("test")


@run_command(add_cmd, add_cmd_list, add_cmd_incomplete,
             add_cmd_keyboard, add_cmd_exception,
             add_cmd_prepare, add_cmd_purge)
def run_cmd(cmds: commands) -> int:
    cmds.logger.debug("main")
    return 0


@pre_command(run_cmd)
def pre_cmd(cmds: commands) -> int:
    cmds.logger.debug("prepare")
    return 0


@end_command(run_cmd)
def end_cmd(cmds: commands) -> int:
    cmds.logger.debug("purge")
    return 0


class test_commands(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        self.cmds = commands()

    def tearDown(self):
        pass

    @mock.patch.object(sys, "exit")
    def test_help_action(self, mock_exit: mock.Mock):
        mock_exit.side_effect = [Exception("xkits-test")]
        self.assertRaises(Exception, self.cmds.run, add_cmd, "--help".split())
        mock_exit.assert_called_once_with(0)

    @mock.patch.object(sys, "exit")
    def test_help_action_h(self, mock_exit: mock.Mock):
        mock_exit.side_effect = [Exception("xkits-test")]
        self.assertRaises(Exception, self.cmds.run, add_cmd, "-h".split())
        mock_exit.assert_called_once_with(0)

    @mock.patch.object(sys, "exit")
    def test_version_action(self, mock_exit: mock.Mock):
        self.cmds.version = __version__
        mock_exit.side_effect = [Exception("xkits-test")]
        self.assertRaises(Exception, self.cmds.run, add_cmd,
                          "--version".split())
        mock_exit.assert_called_once_with(0)

    def test_version_debug(self):
        self.cmds.version = __version__
        ret = self.cmds.run(argv="--stderr --debug".split())
        self.assertEqual(ret, 0)

    def test_root(self):
        self.assertIsInstance(self.cmds.root, add_command)
        assert isinstance(self.cmds.root, add_command)
        self.assertIs(self.cmds.root.cmds, self.cmds)
        self.assertIsInstance(self.cmds.root.bind, run_command)
        assert isinstance(self.cmds.root.bind, run_command)
        self.assertIsInstance(self.cmds.root.bind.bind, add_command)
        self.assertIs(self.cmds.root.bind.bind, self.cmds.root)
        if self.cmds.root.bind.prep:
            self.assertIsInstance(self.cmds.root.bind.prep, pre_command)
            self.assertIs(self.cmds.root.bind.prep.main, self.cmds.root.bind)
        if self.cmds.root.bind.done:
            self.assertIsInstance(self.cmds.root.bind.done, end_command)
            self.assertIs(self.cmds.root.bind.done.main, self.cmds.root.bind)
        self.assertIsInstance(self.cmds.root.subs, Tuple)
        assert isinstance(self.cmds.root.subs, Tuple)
        for _sub in self.cmds.root.subs:
            self.assertIsInstance(_sub, add_command)
            self.assertIs(_sub.root, self.cmds.root)
            self.assertIs(_sub.prev, self.cmds.root)
            self.assertIsInstance(_sub.bind, run_command)
            assert isinstance(_sub.bind, run_command)
            if _sub.bind.prep:
                self.assertIsInstance(_sub.bind.prep, pre_command)
                self.assertIs(_sub.bind.prep.main, _sub.bind)
            if _sub.bind.done:
                self.assertIsInstance(_sub.bind.done, end_command)
                self.assertIs(_sub.bind.done.main, _sub.bind)
            if _sub.subs:
                for _son in _sub.subs:
                    self.assertIsInstance(_son, add_command)
                    self.assertIs(_son.root, self.cmds.root)
                    self.assertIs(_son.prev, _sub)

    def test_parse_root(self):
        ret = self.cmds.parse(argv="--stdout --debug".split())
        self.assertIsInstance(ret, Namespace)

    def test_run_root_error(self):
        ret = self.cmds.run(mock.Mock(), argv="--stdout -d".split())
        self.assertEqual(ret, ENOENT)

    def test_run_KeyboardInterrupt(self):
        ret = self.cmds.run(argv="keyboard --stderr -d".split())
        self.assertEqual(ret, ECANCELED)

    def test_run_BaseException(self):
        with TemporaryDirectory() as _tmp:
            path = os.path.join(_tmp, "log.txt")
            ret = self.cmds.run(argv=f"exception --log {path}".split())
            self.assertEqual(ret, ENOTRECOVERABLE)

    def test_run_prepare(self):
        ret = self.cmds.run(argv="prepare --stdout --debug".split())
        self.assertEqual(ret, -1)

    def test_run_purge(self):
        ret = self.cmds.run(argv="purge --stderr --debug".split())
        self.assertEqual(ret, 1)

    @mock.patch.object(argp, "filter_optional_name")
    def test_filter_optional_name(self, mock_filter_optional_name: mock.Mock):
        mock_filter_optional_name.return_value = []
        ret = self.cmds.run(add_cmd, [], prog="example",
                            description="unittest")
        self.assertEqual(ret, 0)

    def test_has_sub(self):
        ret = self.cmds.has_sub(add_cmd)
        self.assertIsInstance(ret, bool)

    def test_subcommand_list(self):
        ret = self.cmds.run(add_cmd, "incomplete".split(), prog="example")
        self.assertEqual(ret, -1)


class test_commands_logger(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        self.cmds = commands()

    def tearDown(self):
        pass

    def test_disable_logger(self):
        self.cmds.enabled_logger = False
        ret = self.cmds.run(argv=[])
        self.assertEqual(ret, 0)


if __name__ == "__main__":
    unittest.main()



# coding:utf-8

import unittest

from xkits import logger
import mock


class test_log(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        self.log = logger.log()

    def tearDown(self):
        pass

    def test_initiate_logger(self):
        self.log.initiate_logger(self.log.get_logger())

    @mock.patch.object(logger.os, "makedirs", mock.MagicMock())
    @mock.patch.object(logger.os.path, "isdir", mock.MagicMock(side_effect=[True]))  # noqa:E501
    @mock.patch.object(logger.os.path, "exists", mock.MagicMock(side_effect=[False]))  # noqa:E501
    @mock.patch.object(logger.os.path, "dirname", mock.MagicMock())
    @mock.patch.object(logger.logging, "FileHandler", mock.MagicMock())
    def test_new_file_handler(self):
        self.log.new_file_handler("unittest.log")


if __name__ == "__main__":
    unittest.main()
